@@ -5,7 +5,9 @@ import hashlib
 import json
 import os
 import pathlib
+import re
 import shutil
+import tarfile
 import tempfile
 import uuid
 import zipfile
@@ -330,6 +332,46 @@ def create_file(path: PathIn, content: str = "", *, overwrite: bool = False) ->
     write_file(path, content)
 
 
+def create_tar_file(
+    path: PathIn,
+    content_paths: list[PathIn],
+    *,
+    overwrite: bool = True,
+    compression: str = "",
+) -> None:
+    """
+    Create tar file at path compressing directories/files listed in content_paths.
+    If overwrite is allowed and dest tar already exists, it will be overwritten.
+    """
+    path = _get_path(path)
+    assert_not_dir(path)
+    if not overwrite:
+        assert_not_exists(path)
+    make_dirs_for_file(path)
+
+    def _write_content_to_tar_file(
+        file: tarfile.TarFile, content_path: PathIn, basedir: str = ""
+    ) -> None:
+        path = _get_path(content_path)
+        assert_exists(path)
+        if is_file(path):
+            filename = get_filename(path)
+            filepath = join_path(basedir, filename)
+            file.add(path, filepath)
+        elif is_dir(path):
+            for item_name in os.listdir(path):
+                item_path = join_path(path, item_name)
+                item_basedir = (
+                    join_path(basedir, item_name) if is_dir(item_path) else basedir
+                )
+                _write_content_to_tar_file(file, item_path, item_basedir)
+
+    compression = f"w:{compression}" if compression else "w"
+    with tarfile.open(path, compression) as file:
+        for content_path in content_paths:
+            _write_content_to_tar_file(file, content_path)
+
+
 def create_zip_file(
     path: PathIn,
     content_paths: list[PathIn],
@@ -418,18 +460,39 @@ def download_file(
     Download a file from url to dirpath.
     If dirpath is not provided, the file will be downloaded to a temp directory.
     If filename is provided, the file will be named using filename.
-    It is possible to pass extra request options (eg. for authentication) using **kwargs.
+    It is possible to pass extra request options
+    (eg. for authentication) using **kwargs.
     """
     _require_requests_installed()
     # https://stackoverflow.com/a/16696317/2096218
-    dirpath = dirpath or tempfile.gettempdir()
-    dirpath = _get_path(dirpath)
-    filename = filename or get_filename(url) or "download"
-    filepath = join_path(dirpath, filename)
-    make_dirs_for_file(filepath)
+
     kwargs["stream"] = True
     with requests.get(url, **kwargs) as response:
         response.raise_for_status()
+
+        # build filename
+        if not filename:
+            # detect filename from headers
+            content_disposition = response.headers.get("content-disposition", "") or ""
+            filename_pattern = r'filename="(.*)"'
+            filename_match = re.search(filename_pattern, content_disposition)
+            if filename_match:
+                filename = filename_match.group(1)
+            # or detect filename from url
+            if not filename:
+                filename = get_filename(url)
+            # or fallback to a unique name
+            if not filename:
+                filename_uuid = str(uuid.uuid4())
+                filename = f"download-{filename_uuid}"
+
+        # build filepath
+        dirpath = dirpath or tempfile.gettempdir()
+        dirpath = _get_path(dirpath)
+        filepath = join_path(dirpath, filename)
+        make_dirs_for_file(filepath)
+
+        # write file to disk
         with open(filepath, "wb") as file:
             for chunk in response.iter_content(chunk_size=chunk_size):
                 if chunk:
@@ -445,6 +508,30 @@ def exists(path: PathIn) -> bool:
     return os.path.exists(path)
 
 
+def extract_tar_file(
+    path: PathIn,
+    dest: PathIn,
+    *,
+    autodelete: bool = False,
+    content_paths: Iterable[tarfile.TarInfo] | None = None,
+) -> None:
+    """
+    Extract tar file at path to dest path.
+    If autodelete, the archive will be deleted after extraction.
+    If content_paths list is defined,
+    only listed items will be extracted, otherwise all.
+    """
+    path = _get_path(path)
+    dest = _get_path(dest)
+    assert_file(path)
+    assert_not_file(dest)
+    make_dirs(dest)
+    with tarfile.TarFile(path, "r") as file:
+        file.extractall(dest, members=content_paths)
+    if autodelete:
+        remove_file(path)
+
+
 def extract_zip_file(
     path: PathIn,
     dest: PathIn,
@@ -455,7 +542,8 @@ def extract_zip_file(
     """
     Extract zip file at path to dest path.
     If autodelete, the archive will be deleted after extraction.
-    If content_paths list is defined, only listed items will be extracted, otherwise all.
+    If content_paths list is defined,
+    only listed items will be extracted, otherwise all.
     """
     path = _get_path(path)
     dest = _get_path(dest)
@@ -476,7 +564,8 @@ def _filter_paths(
 ) -> list[str]:
     """
     Filter paths relative to basepath according to the optional predicate function.
-    If predicate is defined, paths are filtered using it, otherwise all paths will be listed.
+    If predicate is defined, paths are filtered using it,
+    otherwise all paths will be listed.
     """
     paths = []
     for relpath in relpaths:
