@@ -333,6 +333,28 @@ class fsutil_test_case(unittest.TestCase):
         self.assertTrue(fsutil.is_file(zip_path))
         self.assertTrue(fsutil.get_file_size(zip_path) > 0)
 
+    def test_create_tar_file(self):
+        tar_path = self.temp_path("archive.zip")
+        f1_path = self.temp_path("a/b/f1.txt")
+        f2_path = self.temp_path("a/b/f2.txt")
+        f3_path = self.temp_path("x/y/f3.txt")
+        f4_path = self.temp_path("x/y/f4.txt")
+        fsutil.create_file(f1_path, content="hello world 1")
+        fsutil.create_file(f2_path, content="hello world 2")
+        fsutil.create_file(f3_path, content="hello world 3")
+        fsutil.create_file(f4_path, content="hello world 4")
+        fsutil.create_tar_file(tar_path, [f1_path, f2_path, f3_path, f4_path])
+        with self.assertRaises(OSError):
+            fsutil.create_tar_file(
+                tar_path, [f1_path, f2_path, f3_path, f4_path], overwrite=False
+            )
+        self.assertTrue(fsutil.is_file(f1_path))
+        self.assertTrue(fsutil.is_file(f2_path))
+        self.assertTrue(fsutil.is_file(f3_path))
+        self.assertTrue(fsutil.is_file(f4_path))
+        self.assertTrue(fsutil.is_file(tar_path))
+        self.assertTrue(fsutil.get_file_size(tar_path) > 0)
+
     def test_delete_dir(self):
         fsutil.create_file(self.temp_path("a/b/c/d.txt"))
         fsutil.create_file(self.temp_path("a/b/c/e.txt"))
@@ -401,8 +423,8 @@ class fsutil_test_case(unittest.TestCase):
         self.assertFalse(fsutil.exists(path4))
 
     def test_download_file(self):
-        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/master/README.md"
-        path = fsutil.download_file(url, __file__)
+        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/main/README.md"
+        path = fsutil.download_file(url, dirpath=__file__)
         self.assertTrue(fsutil.exists(path))
         lines = fsutil.read_file_lines(path, skip_empty=False)
         lines_count = len(lines)
@@ -411,8 +433,8 @@ class fsutil_test_case(unittest.TestCase):
         self.assertFalse(fsutil.exists(path))
 
     def test_download_file_multiple_to_temp_dir(self):
-        for i in range(3):
-            url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/master/README.md"
+        for _ in range(3):
+            url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/main/README.md"
             path = fsutil.download_file(url)
             self.assertTrue(fsutil.exists(path))
             lines = fsutil.read_file_lines(path, skip_empty=False)
@@ -424,9 +446,9 @@ class fsutil_test_case(unittest.TestCase):
     def test_download_file_without_requests_installed(self):
         requests_installed = fsutil.requests_installed
         fsutil.requests_installed = False
-        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/master/README.md"
+        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/main/README.md"
         with self.assertRaises(ModuleNotFoundError):
-            fsutil.download_file(url, __file__)
+            fsutil.download_file(url, dirpath=__file__)
         fsutil.requests_installed = requests_installed
 
     def test_exists(self):
@@ -479,6 +501,46 @@ class fsutil_test_case(unittest.TestCase):
         self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f1.txt")))
         self.assertFalse(fsutil.is_file(zip_path))
 
+    def test_extract_tar_file(self):
+        tar_path = self.temp_path("archive.tar")
+        untar_path = self.temp_path("unarchive/")
+        f1_path = self.temp_path("a/b/f1.txt")
+        f2_path = self.temp_path("a/b/f2.txt")
+        f3_path = self.temp_path("j/k/f3.txt")
+        f4_path = self.temp_path("j/k/f4.txt")
+        f5_path = self.temp_path("x/y/z/f5.txt")
+        f6_path = self.temp_path("x/y/z/f6.txt")
+        f5_f6_dir = self.temp_path("x")
+        fsutil.create_file(f1_path, content="hello world 1")
+        fsutil.create_file(f2_path, content="hello world 2")
+        fsutil.create_file(f3_path, content="hello world 3")
+        fsutil.create_file(f4_path, content="hello world 4")
+        fsutil.create_file(f5_path, content="hello world 5")
+        fsutil.create_file(f6_path, content="hello world 6")
+        fsutil.create_tar_file(
+            tar_path, [f1_path, f2_path, f3_path, f4_path, f5_f6_dir]
+        )
+        fsutil.extract_tar_file(tar_path, untar_path)
+        self.assertTrue(fsutil.is_dir(untar_path))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f1.txt")))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f2.txt")))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f3.txt")))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f4.txt")))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/y/z/f5.txt")))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/y/z/f6.txt")))
+        self.assertTrue(fsutil.is_file(tar_path))
+
+    def test_extract_tar_file_with_autodelete(self):
+        tar_path = self.temp_path("archive.tar")
+        untar_path = self.temp_path("unarchive/")
+        path = self.temp_path("f1.txt")
+        fsutil.create_file(path, content="hello world 1")
+        fsutil.create_tar_file(tar_path, [path])
+        fsutil.extract_tar_file(tar_path, untar_path, autodelete=True)
+        self.assertTrue(fsutil.is_dir(untar_path))
+        self.assertTrue(fsutil.is_file(self.temp_path("unarchive/f1.txt")))
+        self.assertFalse(fsutil.is_file(tar_path))
+
     def test_get_dir_creation_date(self):
         path = self.temp_path("a/b/c.txt")
         fsutil.create_file(path, content="Hello World")
@@ -661,19 +723,19 @@ class fsutil_test_case(unittest.TestCase):
         s = "/root/a/b/c/Document.txt"
         self.assertEqual(fsutil.get_parent_dir(s), "/root/a/b/c")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 0), "/root/a/b/c")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=0), "/root/a/b/c")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 1), "/root/a/b/c")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=1), "/root/a/b/c")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 2), "/root/a/b")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=2), "/root/a/b")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 3), "/root/a")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=3), "/root/a")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 4), "/root")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=4), "/root")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 5), "/")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=5), "/")
         s = "/root/a/b/c/Document.txt"
-        self.assertEqual(fsutil.get_parent_dir(s, 6), "/")
+        self.assertEqual(fsutil.get_parent_dir(s, levels=6), "/")
 
     def test_get_unique_name(self):
         path = self.temp_path("a/b/c")
@@ -780,7 +842,7 @@ class fsutil_test_case(unittest.TestCase):
 
     def test_make_dirs_race_condition(self):
         path = self.temp_path("a/b/c/")
-        for i in range(0, 20):
+        for _ in range(0, 20):
             t = threading.Thread(target=fsutil.make_dirs, args=[path], kwargs={})
             t.start()
         t.join()
@@ -819,6 +881,11 @@ class fsutil_test_case(unittest.TestCase):
         with self.assertRaises(OSError):
             fsutil.make_dirs_for_file(path)
 
+    def test_make_dirs_for_file_with_filename_only(self):
+        path = "document.txt"
+        fsutil.make_dirs_for_file(path)
+        self.assertFalse(fsutil.is_file(path))
+
     def test_move_dir(self):
         path = self.temp_path("a/b/c.txt")
         fsutil.create_file(path, content="Hello World")
@@ -840,7 +907,7 @@ class fsutil_test_case(unittest.TestCase):
         self.assertEqual(fsutil.read_file(path), "Hello World")
 
     def test_read_file_from_url(self):
-        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/master/README.md"
+        url = "https://raw.githubusercontent.com/fabiocaccamo/python-fsutil/main/README.md"
         content = fsutil.read_file_from_url(url)
         self.assertTrue("python-fsutil" in content)
 
@@ -850,7 +917,7 @@ class fsutil_test_case(unittest.TestCase):
         data = {
             "test": "Hello World",
             "test_datetime": now,
-            "test_set": {1, 1, 2, 2, 3, 3},
+            "test_set": {1, 2, 3},
         }
         fsutil.write_file_json(self.temp_path("a/b/c.json"), data=data)
         expected_data = data.copy()
@@ -1132,6 +1199,10 @@ class fsutil_test_case(unittest.TestCase):
         s = "/root/a/b/c/Document.txt"
         self.assertEqual(fsutil.split_filepath(s), ("/root/a/b/c", "Document.txt"))
 
+    def test_split_filepath_with_filename_only(self):
+        s = "Document.txt"
+        self.assertEqual(fsutil.split_filepath(s), ("", "Document.txt"))
+
     def test_split_path(self):
         s = "/root/a/b/c/Document.txt"
         self.assertEqual(fsutil.split_path(s), ["root", "a", "b", "c", "Document.txt"])
@@ -1143,6 +1214,13 @@ class fsutil_test_case(unittest.TestCase):
         fsutil.write_file(self.temp_path("a/b/c.txt"), content="Hello Jupiter")
         self.assertEqual(fsutil.read_file(path), "Hello Jupiter")
 
+    def test_write_file_with_filename_only(self):
+        path = "document.txt"
+        fsutil.write_file(path, content="Hello World")
+        self.assertTrue(fsutil.is_file(path))
+        # cleanup
+        fsutil.remove_file(path)
+
     def test_write_file_json(self):
         path = self.temp_path("a/b/c.json")
         now = datetime.now()
@@ -1155,7 +1233,13 @@ class fsutil_test_case(unittest.TestCase):
         fsutil.write_file_json(self.temp_path("a/b/c.json"), data=data)
         self.assertEqual(
             fsutil.read_file(path),
-            f"""{{"test": "Hello World", "test_datetime": "{now.isoformat()}", "test_decimal": "{dec}"}}""",
+            (
+                "{"
+                f'"test": "Hello World", '
+                f'"test_datetime": "{now.isoformat()}", '
+                f'"test_decimal": "{dec}"'
+                "}"
+            ),
         )
 
     def test_write_file_with_append(self):
